package com.example.webapp.controller;

import com.example.webapp.dto.ContractInput;
import com.example.webapp.entity.Contract;
import com.example.webapp.service.ContractService;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import java.time.LocalDate;
import java.util.List;
import java.util.Optional;

@Controller
@RequestMapping("contract")
public class ContractController {

    private final ContractService contractService;

    public ContractController(ContractService contractService) {
        this.contractService = contractService;
    }

    @GetMapping
    String list(Model model) {
        List<Contract> contracts = contractService.findAll();
        model.addAttribute("contracts", contracts);
        return "contract/list";
    }

    @GetMapping("create")
    String showCreate(@ModelAttribute ContractInput contractInput) {
        return "contract/create";
    }

    @PostMapping("create")
    String create(@ModelAttribute @Validated ContractInput contractInput, BindingResult result) {
        if (result.hasErrors()) {
            return "contract/create";
        }
        Optional<String> msgOpt = contractService.create(contractInput);
        if (msgOpt.isPresent()) {
            result.reject("contractInput.global", msgOpt.get());
            return "contract/create";
        }
        return "redirect:/contract";
    }

    @GetMapping("edit")
    ModelAndView showEdit(@RequestParam String phoneNumber, @RequestParam LocalDate startDate) {
        Optional<Contract> contractOpt = contractService.findOne(phoneNumber, startDate);
        if (contractOpt.isPresent()) {
            Contract contract = contractOpt.get();
            return new ModelAndView("contract/edit", "contractInput",
                    new ContractInput(contract.getPhoneNumber(), contract.getStartDate(), contract.getEndDate(),
                            contract.getChargeRule()));
        } else {
            return new ModelAndView("redirect:/notfound");
        }
    }

    @PostMapping("update")
    String update(@ModelAttribute @Validated ContractInput contractInput, BindingResult result) {
        if (result.hasErrors()) {
            return "contract/edit";
        }
        Optional<String> msgOpt = contractService.update(contractInput);
        if (msgOpt.isPresent()) {
            result.reject("contractInput.global", msgOpt.get());
            return "contract/edit";
        }
        return "redirect:/contract";
    }

    @PostMapping("delete")
    String delete(@RequestParam String phoneNumber, @RequestParam LocalDate startDate) {
        contractService.delete(phoneNumber, startDate);
        return "redirect:/contract";
    }

}
